#ifndef __G69DDCAPI_H__
#define __G69DDCAPI_H__

#include <stdint.h>

#pragma pack(push,1)

#define G69DDC_CLASS_ID_DEVICE_ENUMERATOR	            0x0009F001
#define G69DDC_CLASS_ID_DEVICE			                0x0009F002
#define G69DDC_CLASS_ID_NETWORK_ENUMERATOR              0x0009F003

typedef int (*G69DDC_CREATE_INSTANCE)(uint32_t ClassId,void **Interface);

#define G69DDC_OPEN_DEMO                                ((char*)0)
#define G69DDC_OPEN_FIRST                               ((char*)-1)

#define G69DDC_INTERFACE_TYPE_PCIE                      0x00
#define G69DDC_INTERFACE_TYPE_USB2                      0x01
#define G69DDC_INTERFACE_TYPE_USB3                      0x02
#define G69DDC_INTERFACE_TYPE_LAN                       0x03
#define G69DDC_INTERFACE_TYPE_DEMO                      0xFF

#define G69DDC_FRONT_PANEL_LED_MODE_OFF                 0
#define G69DDC_FRONT_PANEL_LED_MODE_ON                  1
#define G69DDC_FRONT_PANEL_LED_MODE_DIAG                2

#define G69DDC_DEVICE_STATE_HIGH_TEMPERATURE            0x00000001

#define G69DDC_TEMPERATURE_NA                           (-0x4000)

#define G69DDC_MODE_AM                                  0
#define G69DDC_MODE_AMS                                 1
#define G69DDC_MODE_LSB                                 2
#define G69DDC_MODE_USB                                 3
#define G69DDC_MODE_DSB                                 4
#define G69DDC_MODE_ISB                                 5
#define G69DDC_MODE_CW                                  6
#define G69DDC_MODE_FM                                  7
#define G69DDC_MODE_FMW                                 8
#define G69DDC_MODE_COUNT                               (G69DDC_MODE_FMW+1)

#define G69DDC_SIDE_BAND_LOWER                          1
#define G69DDC_SIDE_BAND_UPPER                          2
#define G69DDC_SIDE_BAND_BOTH                           3

#define G69DDC_AUDIO_STREAM_CALLBACK_STAGE_0            0
#define G69DDC_AUDIO_STREAM_CALLBACK_STAGE_1            1
#define G69DDC_AUDIO_STREAM_CALLBACK_STAGE_2            2

#define G69DDC_DEMODULATOR_PARAM_AMS_SIDE_BAND          0x00000001
#define G69DDC_DEMODULATOR_PARAM_AMS_CAPTURE_RANGE      0x00000002
#define G69DDC_DEMODULATOR_PARAM_CW_FREQUENCY           0x00000003
#define G69DDC_DEMODULATOR_PARAM_DSB_SIDE_BAND          0x00000004
#define G69DDC_DEMODULATOR_PARAM_ISB_SIDE_BAND          0x00000005
#define G69DDC_DEMODULATOR_PARAM_FMW_STEREO             0x00000006
#define G69DDC_DEMODULATOR_PARAM_FMW_DEEMPHASIS         0x00000007

#define G69DDC_DEMODULATOR_STATE_AMS_LOCK               0x00000001
#define G69DDC_DEMODULATOR_STATE_AMS_FREQUENCY          0x00000002
#define G69DDC_DEMODULATOR_STATE_AM_DEPTH               0x00000003
#define G69DDC_DEMODULATOR_STATE_TUNE_ERROR             0x00000004
#define G69DDC_DEMODULATOR_STATE_FM_DEVIATION           0x00000005
#define G69DDC_DEMODULATOR_STATE_DSB_LOCK               0x00000006
#define G69DDC_DEMODULATOR_STATE_DSB_FREQUENCY          0x00000007
#define G69DDC_DEMODULATOR_STATE_FMW_STEREO             0x00000008

#define G69DDC_ADC_SAMPLES_PER_SNAPSHOT_64K             0x10000
#define G69DDC_ADC_SAMPLES_PER_SNAPSHOT_128K            0x20000
#define G69DDC_ADC_SAMPLES_PER_SNAPSHOT_256K            0x40000

#define G69DDC_SWEEPER_STEP_2MHZ                        2000000
#define G69DDC_SWEEPER_STEP_6MHZ                        6000000
#define G69DDC_SWEEPER_STEP_10MHZ                       10000000
#define G69DDC_SWEEPER_STEP_30MHZ                       30000000

#define G69DDC_FLAGS_EXTERNAL_REFERENCE_IN              0x00000001
#define G69DDC_FLAGS_COHERENT                           0x00000004
#define G69DDC_FLAGS_EXTERNAL_REFERENCE_OUT             0x00000008
#define G69DDC_FLAGS_1PPS                               0x00000010
#define G69DDC_FLAGS_FCC                                0x80000000

typedef struct
{
    char            DevicePath[256];
    uint8_t         InterfaceType;
    char            SerialNumber[9];
    uint16_t        HWVersion;
    uint16_t        FWVersion[3];
    uint8_t         EEPROMVersion;

    struct
    {
        uint32_t    Bandwidth;
    }               Range1;

    struct
    {
        uint32_t    MinFrequency;
        uint64_t    MaxFrequency;
        uint32_t    Step;
        uint32_t    Bandwidth;
    }               Range2;

    uint32_t        MaxDDC1ChannelCount;
    uint32_t        MaxDDC2ChannelCount;
    uint32_t        MaxDDC1TypeCount;
    uint32_t        MaxDDC2TypeCount;
    uint32_t        MaxDDC2ChannelsPerDDC1Channel;

    uint32_t        Flags;

    uint8_t         MACAddress[6];
} G69DDC_DEVICE_INFO;

typedef struct
{
    uint32_t    Flags;
    int32_t     Temperatures[3];
    uint32_t    FanRPM;
    uint64_t    DataTransferred;
    uint64_t    DataLost;
} G69DDC_DEVICE_STATE;

typedef struct
{
    uint32_t    SampleRate;
    uint32_t    Bandwidth;
    uint32_t    BitsPerSample;
} G69DDC_DDC_INFO;

typedef struct
{
    uint32_t    Tune;   //Hz
    uint32_t    Lock;   //Hz
} G69DDC_AMS_CAPTURE_RANGE;

typedef struct
{
    uint16_t    Size;
    uint32_t    IpAddress;  //network-byte-order
    uint32_t    Mask;       //network-byte-order
} G69DDC_NETWORK_PARAMS;

#ifdef __cplusplus

#define G69DDC_DECLARE_INTERFACE(__name__) struct __name__
#define G69DDC_DECLARE_INTERFACE_2(__name__,__base__) struct __name__ : public __base__
#define G69DDC_THIS
#define G69DDC_METHOD(__type__,__method__,__params__) virtual __type__ __method__ __params__=0
#define G69DDC_METHOD_NO_PARAMS(__type__,__method__) virtual __type__ __method__(void)=0
#else //__cplusplus
#define G69DDC_THIS struct G69DDC_INTERFACE *This,
#define G69DDC_DECLARE_INTERFACE(__name__) typedef struct __name__ { const struct __name__##Vtbl *lpVtbl;} __name__;\
    typedef const struct __name__##Vtbl __name__##Vtbl;struct __name__##Vtbl
#define G69DDC_DECLARE_INTERFACE_2(__name__,__base__) G69DDC_DECLARE_INTERFACE(__name__)
#define G69DDC_METHOD(__type__,__method__,__params__) __type__ (*__method__)__params__
#define G69DDC_METHOD_NO_PARAMS(__type__,__method__) __type__ (*__method__)(struct G69DDC_INTERFACE *This)
#endif //__cplusplus

#undef G69DDC_INTERFACE
#define G69DDC_INTERFACE IG69DDCDeviceEnumerator

G69DDC_DECLARE_INTERFACE(IG69DDCDeviceEnumerator)
{
    G69DDC_METHOD_NO_PARAMS(void,Free);
    G69DDC_METHOD_NO_PARAMS(int,Enumerate);
    G69DDC_METHOD_NO_PARAMS(uint32_t,GetCount);
    G69DDC_METHOD(int,GetDeviceInfo,(G69DDC_THIS uint32_t DeviceIndex,G69DDC_DEVICE_INFO *DeviceInfo));
};


#undef G69DDC_INTERFACE
#define G69DDC_INTERFACE IG69DDCNetworkEnumeratorCallback

G69DDC_DECLARE_INTERFACE(IG69DDCNetworkEnumeratorCallback)
{
    G69DDC_METHOD(void,G69DDC_DeviceFound,(G69DDC_THIS uint32_t IpAddress,uint16_t Port,const char *SerialNumber,uint32_t Flags));
};

#undef G69DDC_INTERFACE
#define G69DDC_INTERFACE IG69DDCNetworkEnumerator

G69DDC_DECLARE_INTERFACE(IG69DDCNetworkEnumerator)
{
    G69DDC_METHOD_NO_PARAMS(void,Free);
    G69DDC_METHOD(int,Start,(G69DDC_THIS IG69DDCNetworkEnumeratorCallback *Callback));
    G69DDC_METHOD_NO_PARAMS(void,Stop);
};

struct IG69DDCDevice;

#undef G69DDC_INTERFACE
#define G69DDC_INTERFACE IG69DDCDeviceCallback

G69DDC_DECLARE_INTERFACE(IG69DDCDeviceCallback)
{
    G69DDC_METHOD(void,G69DDC_ADCSnapshotCallback,(G69DDC_THIS struct IG69DDCDevice *Device,const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel));
    G69DDC_METHOD(void,G69DDC_DDC1StreamCallback,(G69DDC_THIS struct IG69DDCDevice *Device,uint32_t DDC1ChannelId,const void *Buffer,uint32_t Count,uint32_t BitsPerSample));
    G69DDC_METHOD(int,G69DDC_DDC1PlaybackStreamCallback,(G69DDC_THIS struct IG69DDCDevice *Device,uint32_t DDC1ChannelId,void *Buffer,uint32_t Count,uint32_t BitsPerSample));
    G69DDC_METHOD(void,G69DDC_DDC2StreamCallback,(G69DDC_THIS struct IG69DDCDevice *Device,uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count));
    G69DDC_METHOD(void,G69DDC_DDC2PreprocessedStreamCallback,(G69DDC_THIS struct IG69DDCDevice *Device,uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,float Peak,float RMS));
    G69DDC_METHOD(void,G69DDC_AudioStreamCallback,(G69DDC_THIS struct IG69DDCDevice *Device,uint32_t DDC2ChannelId,uint32_t Stage,const float *Buffer,uint32_t Count));
    G69DDC_METHOD(int,G69DDC_AudioPlaybackStreamCallback,(G69DDC_THIS struct IG69DDCDevice *Device,uint32_t DDC2ChannelId,float *Buffer,uint32_t Count));
    G69DDC_METHOD(int,G69DDC_SweeperCallback,(G69DDC_THIS struct IG69DDCDevice *Device,const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint64_t FrontEndFrequency,uint32_t Repeat));
};

#define G69DDC_DEVICE_INTERFACE_VERSION    0x00000001

#undef G69DDC_INTERFACE
#define G69DDC_INTERFACE IG69DDCDevice

G69DDC_DECLARE_INTERFACE(IG69DDCDevice)
{
    G69DDC_METHOD_NO_PARAMS(int,Free);

    G69DDC_METHOD(void,SetCallback,(G69DDC_THIS IG69DDCDeviceCallback *Callback));
    G69DDC_METHOD_NO_PARAMS(IG69DDCDeviceCallback*,GetCallback);

    G69DDC_METHOD(int,Open,(G69DDC_THIS const char *Path));
    G69DDC_METHOD_NO_PARAMS(int,Close);

    G69DDC_METHOD_NO_PARAMS(int,IsOpen);

    G69DDC_METHOD(int,GetDeviceInfo,(G69DDC_THIS G69DDC_DEVICE_INFO *DeviceInfo));

    G69DDC_METHOD(int,IsConnected,(G69DDC_THIS int *Connected));

    G69DDC_METHOD(int,SetLED,(G69DDC_THIS uint32_t LEDMode));
    G69DDC_METHOD(int,GetLED,(G69DDC_THIS uint32_t *LEDMode));

    G69DDC_METHOD(int,SetPower,(G69DDC_THIS int Power));
    G69DDC_METHOD(int,GetPower,(G69DDC_THIS int *Power));

    G69DDC_METHOD(int,GetDeviceState,(G69DDC_THIS G69DDC_DEVICE_STATE *State));

    G69DDC_METHOD(int,SetExternalReference,(G69DDC_THIS int Enabled));
    G69DDC_METHOD(int,GetExternalReference,(G69DDC_THIS int *Enabled));

    G69DDC_METHOD(int,SetBuiltInTest,(G69DDC_THIS int Enabled));
    G69DDC_METHOD(int,GetBuiltInTest,(G69DDC_THIS int *Enabled));

    G69DDC_METHOD(int,SetAttenuator,(G69DDC_THIS uint32_t Attenuator));
    G69DDC_METHOD(int,GetAttenuator,(G69DDC_THIS uint32_t *Attenuator));

    G69DDC_METHOD(int,SetPreamplifier,(G69DDC_THIS int Preamp));
    G69DDC_METHOD(int,GetPreamplifier,(G69DDC_THIS int *Preamp));

    G69DDC_METHOD(int,SetMWFilter,(G69DDC_THIS int Enabled));
    G69DDC_METHOD(int,GetMWFilter,(G69DDC_THIS int *Enabled));

    G69DDC_METHOD(int,SetInverted,(G69DDC_THIS int Inverted));
    G69DDC_METHOD(int,GetInverted,(G69DDC_THIS int *Inverted));

    G69DDC_METHOD(int,SetFrontEndFrequency,(G69DDC_THIS uint64_t Frequency));
    G69DDC_METHOD(int,GetFrontEndFrequency,(G69DDC_THIS uint64_t *Frequency));

    G69DDC_METHOD(int,SetADCNoiseBlanker,(G69DDC_THIS int Enabled));
    G69DDC_METHOD(int,GetADCNoiseBlanker,(G69DDC_THIS int *Enabled));
    G69DDC_METHOD(int,SetADCNoiseBlankerThreshold,(G69DDC_THIS uint16_t Threshold));
    G69DDC_METHOD(int,GetADCNoiseBlankerThreshold,(G69DDC_THIS uint16_t *Threshold));

    G69DDC_METHOD(int,StartADCSnapshots,(G69DDC_THIS uint16_t Interval,uint32_t SamplesPerSnapshot));
    G69DDC_METHOD_NO_PARAMS(int,StopADCSnapshots);

    G69DDC_METHOD(int,GetDDCInfo,(G69DDC_THIS uint32_t DDCTypeIndex,G69DDC_DDC_INFO *DDCInfo));

    G69DDC_METHOD(int,CreateDDC1,(G69DDC_THIS uint32_t *DDC1ChannelId));
    G69DDC_METHOD(int,CreateDDC2,(G69DDC_THIS uint32_t DDC1ChannelId,uint32_t *DDC2ChannelId));

    G69DDC_METHOD(int,DeleteDDC,(G69DDC_THIS uint32_t DDCChannelId));

    G69DDC_METHOD(int,GetDDCCount,(G69DDC_THIS uint32_t DDCChannelId,uint32_t *Count));
    G69DDC_METHOD(int,SetDDC,(G69DDC_THIS uint32_t DDCChannelId,uint32_t TypeIndex));
    G69DDC_METHOD(int,GetDDC,(G69DDC_THIS uint32_t DDCChannelId,uint32_t *TypeIndex,G69DDC_DDC_INFO *DDCInfo));
    G69DDC_METHOD(int,StartDDC,(G69DDC_THIS uint32_t DDCChannelId,uint32_t SampleSetsPerBuffer));
    G69DDC_METHOD(int,StopDDC,(G69DDC_THIS uint32_t DDCChannelId));

    G69DDC_METHOD(int,StartDDC1Playback,(G69DDC_THIS uint32_t DDC1ChannelId,uint32_t SampleSetsPerBuffer,uint32_t BitsPerSample));
    G69DDC_METHOD(int,PauseDDC1Playback,(G69DDC_THIS uint32_t DDC1ChannelId));
    G69DDC_METHOD(int,ResumeDDC1Playback,(G69DDC_THIS uint32_t DDC1ChannelId));

    G69DDC_METHOD(int,SetDDCFrequency,(G69DDC_THIS uint32_t DDCChannelId,int32_t Frequency));
    G69DDC_METHOD(int,GetDDCFrequency,(G69DDC_THIS uint32_t DDCChannelId,int32_t *Frequency));

    G69DDC_METHOD(int,SetDDC2NoiseBlanker,(G69DDC_THIS uint32_t DDC2ChannelId,int Enabled));
    G69DDC_METHOD(int,GetDDC2NoiseBlanker,(G69DDC_THIS uint32_t DDC2ChannelId,int *Enabled));
    G69DDC_METHOD(int,SetDDC2NoiseBlankerThreshold,(G69DDC_THIS uint32_t DDC2ChannelId,double Threshold));
    G69DDC_METHOD(int,GetDDC2NoiseBlankerThreshold,(G69DDC_THIS uint32_t DDC2ChannelId,double *Threshold));
    G69DDC_METHOD(int,GetDDC2NoiseBlankerExcessValue,(G69DDC_THIS uint32_t DDC2ChannelId,double *Value));

    G69DDC_METHOD(int,SetNotchFilter,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int Enabled));
    G69DDC_METHOD(int,GetNotchFilter,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int *Enabled));
    G69DDC_METHOD(int,SetNotchFilterFrequency,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int32_t Frequency));
    G69DDC_METHOD(int,GetNotchFilterFrequency,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int32_t *Frequency));
    G69DDC_METHOD(int,SetNotchFilterBandwidth,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t Bandwidth));
    G69DDC_METHOD(int,GetNotchFilterBandwidth,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t *Bandwidth));
    G69DDC_METHOD(int,SetNotchFilterLength,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t Length));
    G69DDC_METHOD(int,GetNotchFilterLength,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t *Length));

    G69DDC_METHOD(int,SetAGC,(G69DDC_THIS uint32_t DDC2ChannelId,int Enabled));
    G69DDC_METHOD(int,GetAGC,(G69DDC_THIS uint32_t DDC2ChannelId,int *Enabled));
    G69DDC_METHOD(int,SetAGCParams,(G69DDC_THIS uint32_t DDC2ChannelId,double AttackTime,double DecayTime,double ReferenceLevel));
    G69DDC_METHOD(int,GetAGCParams,(G69DDC_THIS uint32_t DDC2ChannelId,double *AttackTime,double *DecayTime,double *ReferenceLevel));
    G69DDC_METHOD(int,SetMaxAGCGain,(G69DDC_THIS uint32_t DDC2ChannelId,double MaxGain));
    G69DDC_METHOD(int,GetMaxAGCGain,(G69DDC_THIS uint32_t DDC2ChannelId,double *MaxGain));
    G69DDC_METHOD(int,SetGain,(G69DDC_THIS uint32_t DDC2ChannelId,double Gain));
    G69DDC_METHOD(int,GetGain,(G69DDC_THIS uint32_t DDC2ChannelID,double *Gain));
    G69DDC_METHOD(int,GetCurrentGain,(G69DDC_THIS uint32_t DDC2ChannelId,double *Gain));

    G69DDC_METHOD(int,GetSignalLevel,(G69DDC_THIS uint32_t DDC2ChannelId,float *Peak,float *RMS));

    G69DDC_METHOD(int,SetDemodulatorMode,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t Mode));
    G69DDC_METHOD(int,GetDemodulatorMode,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t *Mode));
    G69DDC_METHOD(int,SetDemodulatorFrequency,(G69DDC_THIS uint32_t DDC2ChannelId,int32_t Frequency));
    G69DDC_METHOD(int,GetDemodulatorFrequency,(G69DDC_THIS uint32_t DDC2ChannelId,int32_t *Frequency));
    G69DDC_METHOD(int,SetDemodulatorFilterBandwidth,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t Bandwidth));
    G69DDC_METHOD(int,GetDemodulatorFilterBandwidth,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t *Bandwidth));
    G69DDC_METHOD(int,SetDemodulatorFilterShift,(G69DDC_THIS uint32_t DDC2ChannelId,int32_t Shift));
    G69DDC_METHOD(int,GetDemodulatorFilterShift,(G69DDC_THIS uint32_t DDC2ChannelId,int32_t *Shift));
    G69DDC_METHOD(int,SetDemodulatorFilterLength,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t Length));
    G69DDC_METHOD(int,GetDemodulatorFilterLength,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t *Length));

    G69DDC_METHOD(int,SetDemodulatorParam,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t Code,const void *Buffer,uint32_t BufferSize));
    G69DDC_METHOD(int,GetDemodulatorParam,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t Code,void *Buffer,uint32_t BufferSize));
    G69DDC_METHOD(int,GetDemodulatorState,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t Code,void *Buffer,uint32_t BufferSize));

    G69DDC_METHOD(int,GetAudioSampleRateCount,(G69DDC_THIS uint32_t *Count));
    G69DDC_METHOD(int,GetAudioSampleRate,(G69DDC_THIS uint32_t SampleRateIndex,uint32_t *SampleRate));

    G69DDC_METHOD(int,SetAudio,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t SampleRateIndex));
    G69DDC_METHOD(int,GetAudio,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t *SampleRateIndex,uint32_t *SampleRate));

    G69DDC_METHOD(int,StartAudio,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t SampleSetsPerBuffer));
    G69DDC_METHOD(int,StopAudio,(G69DDC_THIS uint32_t DDC2ChannelId));

    G69DDC_METHOD(int,StartAudioPlayback,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t SampleSetsPerBuffer));
    G69DDC_METHOD(int,PauseAudioPlayback,(G69DDC_THIS uint32_t DDC2ChannelId));
    G69DDC_METHOD(int,ResumeAudioPlayback,(G69DDC_THIS uint32_t DDC2ChannelId));

    G69DDC_METHOD(int,SetAudioFilter,(G69DDC_THIS uint32_t DDC2ChannelId,int Enabled));
    G69DDC_METHOD(int,GetAudioFilter,(G69DDC_THIS uint32_t DDC2ChannelId,int *Enabled));
    G69DDC_METHOD(int,SetAudioFilterParams,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis));
    G69DDC_METHOD(int,GetAudioFilterParams,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis));
    G69DDC_METHOD(int,SetAudioFilterLength,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t Length));
    G69DDC_METHOD(int,GetAudioFilterLength,(G69DDC_THIS uint32_t DDC2ChannelId,uint32_t *Length));
    G69DDC_METHOD(int,SetAudioGain,(G69DDC_THIS uint32_t DDC2ChannelId,double Gain));
    G69DDC_METHOD(int,GetAudioGain,(G69DDC_THIS uint32_t DDC2ChannelId,double *Gain));
    G69DDC_METHOD(int,SetVolume,(G69DDC_THIS uint32_t DDC2ChannelId,uint8_t Volume));
    G69DDC_METHOD(int,GetVolume,(G69DDC_THIS uint32_t DDC2ChannelId,uint8_t *Volume));
    G69DDC_METHOD(int,SetMute,(G69DDC_THIS uint32_t DDC2ChannelId,int Mute));
    G69DDC_METHOD(int,GetMute,(G69DDC_THIS uint32_t DDC2ChannelId,int *Mute));

    G69DDC_METHOD(int,SetFrequency,(G69DDC_THIS uint32_t DDC2ChannelId,uint64_t Frequency));
    G69DDC_METHOD(int,GetFrequency,(G69DDC_THIS uint32_t DDC2ChannelId,uint64_t *Frequency));

    G69DDC_METHOD(int,GetSpectrumCompensation,(G69DDC_THIS uint64_t FrontEndFrequency,int32_t Shift,uint32_t Bandwidth,float *Buffer,uint32_t Count));

    G69DDC_METHOD(int,StartSweeper,(G69DDC_THIS uint64_t BeginFrequency,uint64_t EndFrequency,uint64_t StartFrequency,uint32_t Step,int Forward,uint32_t Repeat,uint32_t SamplesPerSnapshot));
    G69DDC_METHOD_NO_PARAMS(int,StopSweeper);

    G69DDC_METHOD(int,SetNetworkParams,(G69DDC_THIS const G69DDC_NETWORK_PARAMS *Params));
    G69DDC_METHOD(int,GetNetworkParams,(G69DDC_THIS G69DDC_NETWORK_PARAMS *Params));

    G69DDC_METHOD(int,GetInterfaceVersion,(G69DDC_THIS uint32_t *Version));

    G69DDC_METHOD(int,Get1PPSCounters,(G69DDC_THIS uint32_t *EventCounter,uint32_t *ADCPeriodCounter));
    G69DDC_METHOD(int,Get1PPSDDC1Counters,(G69DDC_THIS uint32_t DDC1ChannelId,double *SampleCounter,uint64_t *ADCPeriodCounter));
};


typedef void (*G69DDC_ADC_SNAPSHOT_CALLBACK)(const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel,uintptr_t UserData);
typedef void (*G69DDC_DDC1_STREAM_CALLBACK)(uint32_t DDC1ChannelId,const void *Buffer,uint32_t Count,uint32_t BitsPerSample,uintptr_t UserData);
typedef int (*G69DDC_DDC1_PLAYBACK_STREAM_CALLBACK)(uint32_t DDC1ChannelId,void *Buffer,uint32_t Count,uint32_t BitsPerSample,uintptr_t UserData);
typedef void (*G69DDC_DDC2_STREAM_CALLBACK)(uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,uintptr_t UserData);
typedef void (*G69DDC_DDC2_PREPROCESSED_STREAM_CALLBACK)(uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,float Peak,float RMS,uintptr_t UserData);
typedef void (*G69DDC_AUDIO_STREAM_CALLBACK)(uint32_t DDC2ChannelId,uint32_t Stage,const float *Buffer,uint32_t Count,uintptr_t UserData);
typedef int (*G69DDC_AUDIO_PLAYBACK_STREAM_CALLBACK)(uint32_t DDC2ChannelId,float *Buffer,uint32_t Count,uintptr_t UserData);
typedef int (*G69DDC_SWEEPER_CALLBACK)(const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint64_t FrontEndFrequency,uint32_t Repeat,uintptr_t UserData);
typedef int (*G69DDC_SEARCH_NETWORK_DEVICES_CALLBACK)(uint32_t IpAddress,uint16_t Port,const char *SerialNumber,uint32_t Flags,uintptr_t UserData);

typedef struct
{
    G69DDC_ADC_SNAPSHOT_CALLBACK                ADCSnapshotCallback;
    G69DDC_DDC1_STREAM_CALLBACK                 DDC1StreamCallback;
    G69DDC_DDC1_PLAYBACK_STREAM_CALLBACK        DDC1PlaybackStreamCallback;
    G69DDC_DDC2_STREAM_CALLBACK                 DDC2StreamCallback;
    G69DDC_DDC2_PREPROCESSED_STREAM_CALLBACK    DDC2PreprocessedStreamCallback;
    G69DDC_AUDIO_STREAM_CALLBACK                AudioStreamCallback;
    G69DDC_AUDIO_PLAYBACK_STREAM_CALLBACK       AudioPlaybackStreamCallback;
    G69DDC_SWEEPER_CALLBACK                     SweeperCallback;
} G69DDC_CALLBACKS;

#pragma pack(pop)

typedef int (*G69DDC_GET_DEVICE_LIST)(G69DDC_DEVICE_INFO **DeviceList,uint32_t *Count);
typedef void (*G69DDC_FREE_DEVICE_LIST)(G69DDC_DEVICE_INFO *DeviceList);
typedef int (*G69DDC_SEARCH_NETWORK_DEVICES)(G69DDC_SEARCH_NETWORK_DEVICES_CALLBACK Callback,uintptr_t UserData,uint32_t Timeout);
typedef int32_t ( *G69DDC_OPEN_DEVICE)(const char *SerialNumber);
typedef int (*G69DDC_CLOSE_DEVICE)(int32_t hDevice);
typedef int (*G69DDC_IS_DEVICE_CONNECTED)(int32_t hDevice,int *Connected);
typedef int (*G69DDC_SET_CALLBACKS)(int32_t hDevice,const G69DDC_CALLBACKS *Callbacks,uintptr_t UserData);
typedef int (*G69DDC_GET_DEVICE_INFO)(int32_t hDevice,G69DDC_DEVICE_INFO *DeviceInfo);
typedef int (*G69DDC_SET_LED)(uint32_t LEDMode);
typedef int (*G69DDC_GET_LED)(uint32_t *LEDMode);
typedef int (*G69DDC_SET_POWER)(int32_t hDevice,int Power);
typedef int (*G69DDC_GET_POWER)(int32_t hDevice,int *Power);
typedef int (*G69DDC_GET_DEVICE_STATE)(int32_t hDevice,G69DDC_DEVICE_STATE *State);
typedef int (*G69DDC_SET_EXTERNAL_REFERENCE)(int32_t hDevice,int Enabled);
typedef int (*G69DDC_GET_EXTERNAL_REFERENCE)(int32_t hDevice,int *Enabled);
typedef int (*G69DDC_SET_BUILT_IN_TEST)(int32_t hDevice,int Enabled);
typedef int (*G69DDC_GET_BUILT_IN_TEST)(int32_t hDevice,int *Enabled);
typedef int (*G69DDC_SET_ATTENUATOR)(int32_t hDevice,uint32_t Attenuator);
typedef int (*G69DDC_GET_ATTENUATOR)(int32_t hDevice,uint32_t *Attenuator);
typedef int (*G69DDC_SET_PREAMPLIFIER)(int32_t hDevice,int Preamp);
typedef int (*G69DDC_GET_PREAMPLIFIER)(int32_t hDevice,int *Preamp);
typedef int (*G69DDC_SET_MW_FILTER)(int32_t hDevice,int Enabled);
typedef int (*G69DDC_GET_MW_FILTER)(int32_t hDevice,int *Enabled);
typedef int (*G69DDC_SET_FRONT_END_FREQUENCY)(int32_t hDevice,uint64_t Frequency);
typedef int (*G69DDC_GET_FRONT_END_FREQUENCY)(int32_t hDevice,uint64_t *Frequency);
typedef int (*G69DDC_SET_ADC_NOISE_BLANKER)(int32_t hDevice,int Enabled);
typedef int (*G69DDC_GET_ADC_NOISE_BLANKER)(int32_t hDevice,int *Enabled);
typedef int (*G69DDC_SET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDevice,uint16_t Threshold);
typedef int (*G69DDC_GET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDevice,uint16_t *Threshold);
typedef int (*G69DDC_START_ADC_SNAPSHOTS)(int32_t hDevice,uint16_t Interval,uint32_t SamplesPerSnapshot);
typedef int (*G69DDC_STOP_ADC_SNAPSHOTS)(int32_t hDevice);
typedef int (*G69DDC_GET_DDC_INFO)(int32_t hDevice,uint32_t DDCTypeIndex,G69DDC_DDC_INFO *DDCInfo);
typedef int (*G69DDC_CREATE_DDC1)(int32_t hDevice,uint32_t *DDC1ChannelId);
typedef int (*G69DDC_CREATE_DDC2)(int32_t hDevice,uint32_t DDC1ChannelId,uint32_t *DDC2ChannelId);
typedef int (*G69DDC_DELETE_DDC)(int32_t hDevice,uint32_t DDCChannelId);
typedef int (*G69DDC_GET_DDC_COUNT)(int32_t hDevice,uint32_t DDCChannelId,uint32_t *Count);
typedef int (*G69DDC_SET_DDC)(int32_t hDevice,uint32_t DDCChannelId,uint32_t TypeIndex);
typedef int (*G69DDC_GET_DDC)(int32_t hDevice,uint32_t DDCChannelId,uint32_t *TypeIndex,G69DDC_DDC_INFO *DDCInfo);
typedef int (*G69DDC_START_DDC)(int32_t hDevice,uint32_t DDCChannelId,uint32_t SampleSetsPerBuffer);
typedef int (*G69DDC_STOP_DDC)(int32_t hDevice,uint32_t DDCChannelId);
typedef int (*G69DDC_START_DDC1_PLAYBACK)(int32_t hDevice,uint32_t DDC1ChannelId,uint32_t SampleSetsPerBuffer,uint32_t BitsPerSample);
typedef int (*G69DDC_PAUSE_DDC1_PLAYBACK)(int32_t hDevice,uint32_t DDC1ChannelId);
typedef int (*G69DDC_RESUME_DDC1_PLAYBACK)(int32_t hDevice,uint32_t DDC1ChannelId);
typedef int (*G69DDC_SET_DDC_FREQUENCY)(int32_t hDevice,uint32_t DDCChannelId,int32_t Frequency);
typedef int (*G69DDC_GET_DDC_FREQUENCY)(int32_t hDevice,uint32_t DDCChannelId,int32_t *Frequency);
typedef int (*G69DDC_SET_DDC2_NOISE_BLANKER)(int32_t hDevice,uint32_t DDC2ChannelId,int Enabled);
typedef int (*G69DDC_GET_DDC2_NOISE_BLANKER)(int32_t hDevice,uint32_t DDC2ChannelId,int *Enabled);
typedef int (*G69DDC_SET_DDC2_NOISE_BLANKER_THREASHOLD)(int32_t hDevice,uint32_t DDC2ChannelId,double Threshold);
typedef int (*G69DDC_GET_DDC2_NOISE_BLANKER_THREASHOLD)(int32_t hDevice,uint32_t DDC2ChannelId,double *Threshold);
typedef int (*G69DDC_GET_DDC2_NOISE_BLANKER_EXCESS_VALUE)(int32_t hDevice,uint32_t DDC2ChannelId,double *Value);
typedef int (*G69DDC_SET_NOTCH_FILTER)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int Enabled);
typedef int (*G69DDC_GET_NOTCH_FILTER)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int *Enabled);
typedef int (*G69DDC_SET_NOTCH_FILTER_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int32_t Frequency);
typedef int (*G69DDC_GET_NOTCH_FILTER_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int32_t *Frequency);
typedef int (*G69DDC_SET_NOTCH_FILTER_BANDWIDTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t Bandwidth);
typedef int (*G69DDC_GET_NOTCH_FILTER_BANDWIDTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t *Bandwidth);
typedef int (*G69DDC_SET_NOTCH_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t Length);
typedef int (*G69DDC_GET_NOTCH_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t *Length);
typedef int (*G69DDC_SET_AGC)(int32_t hDevice,uint32_t DDC2ChannelId,int Enabled);
typedef int (*G69DDC_GET_AGC)(int32_t hDevice,uint32_t DDC2ChannelId,int *Enabled);
typedef int (*G69DDC_SET_AGC_PARAMS)(int32_t hDevice,uint32_t DDC2ChannelId,double AttackTime,double DecayTime,double ReferenceLevel);
typedef int (*G69DDC_GET_AGC_PARAMS)(int32_t hDevice,uint32_t DDC2ChannelId,double *AttackTime,double *DecayTime,double *ReferenceLevel);
typedef int (*G69DDC_SET_MAX_AGC_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double MaxGain);
typedef int (*G69DDC_GET_MAX_AGC_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double *MaxGain);
typedef int (*G69DDC_SET_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double Gain);
typedef int (*G69DDC_GET_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double *Gain);
typedef int (*G69DDC_GET_CURRENT_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double *Gain);
typedef int (*G69DDC_GET_SIGNAL_LEVEL)(int32_t hDevice,uint32_t DDC2ChannelId,float *Peak,float *RMS);
typedef int (*G69DDC_SET_DEMODULATOR_MODE)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Mode);
typedef int (*G69DDC_GET_DEMODULATOR_MODE)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *Mode);
typedef int (*G69DDC_SET_DEMODULATOR_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t Frequency);
typedef int (*G69DDC_GET_DEMODULATOR_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t *Frequency);
typedef int (*G69DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Bandwidth);
typedef int (*G69DDC_GET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *Bandwidth);
typedef int (*G69DDC_SET_DEMODULATOR_FILTER_SHIFT)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t Shift);
typedef int (*G69DDC_GET_DEMODULATOR_FILTER_SHIFT)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t *Shift);
typedef int (*G69DDC_SET_DEMODULATOR_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Length);
typedef int (*G69DDC_GET_DEMODULATOR_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *Length);
typedef int (*G69DDC_SET_DEMODULATOR_PARAM)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Code,const void *Buffer,uint32_t BufferSize);
typedef int (*G69DDC_GET_DEMODULATOR_PARAM)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Code,void *Buffer,uint32_t BufferSize);
typedef int (*G69DDC_GET_DEMODULATOR_STATE)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Code,void *Buffer,uint32_t BufferSize);
typedef int (*G69DDC_GET_AUDIO_SAMPLE_RATE_COUNT)(int32_t hDevice,uint32_t *Count);
typedef int (*G69DDC_GET_AUDIO_SAMPLE_RATE)(int32_t hDevice,uint32_t SampleRateIndex,uint32_t *SampleRate);
typedef int (*G69DDC_SET_AUDIO)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t SampleRateIndex);
typedef int (*G69DDC_GET_AUDIO)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *SampleRateIndex,uint32_t *SampleRate);
typedef int (*G69DDC_START_AUDIO)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t SampleSetsPerBuffer);
typedef int (*G69DDC_STOP_AUDIO)(int32_t hDevice,uint32_t DDC2ChannelId);
typedef int (*G69DDC_START_AUDIO_PLAYBACK)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t SampleSetsPerBuffer);
typedef int (*G69DDC_PAUSE_AUDIO_PLAYBACK)(int32_t hDevice,uint32_t DDC2ChannelId);
typedef int (*G69DDC_RESUME_AUDIO_PLAYBACK)(int32_t hDevice,uint32_t DDC2ChannelId);
typedef int (*G69DDC_SET_AUDIO_FILTER)(int32_t hDevice,uint32_t DDC2ChannelId,int Enabled);
typedef int (*G69DDC_GET_AUDIO_FILTER)(int32_t hDevice,uint32_t DDC2ChannelId,int *Enabled);
typedef int (*G69DDC_SET_AUDIO_FILTER_PARAMS)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis);
typedef int (*G69DDC_GET_AUDIO_FILTER_PARAMS)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis);
typedef int (*G69DDC_SET_AUDIO_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Length);
typedef int (*G69DDC_GET_AUDIO_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *Length);
typedef int (*G69DDC_SET_AUDIO_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double Gain);
typedef int (*G69DDC_GET_AUDIO_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double *Gain);
typedef int (*G69DDC_SET_VOLUME)(int32_t hDevice,uint32_t DDC2ChannelId,uint8_t Volume);
typedef int (*G69DDC_GET_VOLUME)(int32_t hDevice,uint32_t DDC2ChannelId,uint8_t *Volume);
typedef int (*G69DDC_SET_MUTE)(int32_t hDevice,uint32_t DDC2ChannelId,int Mute);
typedef int (*G69DDC_GET_MUTE)(int32_t hDevice,uint32_t DDC2ChannelId,int *Mute);
typedef int (*G69DDC_SET_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,uint64_t Frequency);
typedef int (*G69DDC_GET_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,uint64_t *Frequency);
typedef int (*G69DDC_GET_SPECTRUM_COMPENSATION)(int32_t hDevice,uint64_t FrontEndFrequency,int32_t Shift,uint32_t Bandwidth,float *Buffer,uint32_t Count);
typedef int (*G69DDC_START_SWEEPER)(int32_t hDevice,uint64_t BeginFrequency,uint64_t EndFrequency,uint64_t StartFrequency,uint32_t Step,int Forward,uint32_t Repeat,uint32_t SamplesPerSnapshot);
typedef int (*G69DDC_STOP_SWEEPER)(int32_t hDevice);
typedef int (*G69DDC_SET_NETWORK_PARAMS)(int32_t hDevice,const G69DDC_NETWORK_PARAMS *Params);
typedef int (*G69DDC_GET_NETWORK_PARAMS)(int32_t hDevice,G69DDC_NETWORK_PARAMS *Params);

#endif //__G69DDCAPI_H__

