/*****************************************************************************/
/*                      WiNRADiO G69DDC EXCELSIOR ULTRA                      */
/*                                SDK DEMO                                   */
/*             Receiver selection and "No receiver found" dialogs            */
/*                        (c) Copyright RADIXON 2022                         */
/*****************************************************************************/

#include "u_receiverselector.h"
#include "u_defs.h"
#include "u_global.h"
#include <arpa/inet.h>

extern G69DDC_CREATE_INSTANCE CreateInstance;

TNoReceiverFoundWindow::TNoReceiverFoundWindow(void):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition)
{
 wxStaticText *t;
 wxSizer *Sizer,*ButtonSizer;

    SetFont(*Global.m_Font);

    Sizer=new wxBoxSizer(wxVERTICAL);

    t=new wxStaticText(this,wxID_ANY,wxT("No G69DDC receiver has been found.\n\n")
                                  wxT("This may mean that either no receiver is connected,\n")
                                  wxT("or it is not powered, or its Windows driver was not\n")
                                  wxT("loaded in time before this application was executed.\n\n")
                                  wxT("In the demo mode, a simulated 69 MHz signal,\n")
                                  wxT("amplitude-modulated with 1 kHz tone, is provided."),
                    wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);


    Sizer->Add(t,1,wxALIGN_TOP|wxALIGN_CENTRE|wxTOP,20);

    ButtonSizer=new wxBoxSizer(wxHORIZONTAL);

    Sizer->Add(ButtonSizer,0,wxALIGN_CENTER|wxALIGN_TOP|wxBOTTOM,20);

    m_BTryAgain=new wxButton(this,wxID_RETRY,wxT("Try again"),wxDefaultPosition,wxSize(65,23));
    ButtonSizer->Add(m_BTryAgain,0,wxALIGN_CENTER);

    ButtonSizer->AddSpacer(4);

    m_BDemo=new wxButton(this,wxID_DEFAULT,wxT("Run in demo mode"),wxDefaultPosition,wxSize(110,23));
    ButtonSizer->Add(m_BDemo,0,wxALIGN_CENTER);

    ButtonSizer->AddSpacer(4);

    m_BNetwork=new wxButton(this,wxID_MORE,wxT("Scan network"),wxDefaultPosition,wxSize(110,23));
    ButtonSizer->Add(m_BNetwork,0,wxALIGN_CENTER);
    ButtonSizer->AddSpacer(4);

    m_BCancel=new wxButton(this,wxID_CANCEL,wxT("Cancel"),wxDefaultPosition,wxSize(65,23));
    ButtonSizer->Add(m_BCancel,0,wxALIGN_CENTER);

    SetSizer(Sizer);

    SetClientSize(400,193);

    Layout();

    CenterOnScreen();
}

TNoReceiverFoundWindow::~TNoReceiverFoundWindow(void)
{
}

void TNoReceiverFoundWindow::OnTryAgain(wxCommandEvent &Event)
{
    EndModal(wxID_RETRY);
}

void TNoReceiverFoundWindow::OnDemo(wxCommandEvent &Event)
{
    EndModal(wxID_DEFAULT);
}

BEGIN_EVENT_TABLE(TNoReceiverFoundWindow, wxDialog)
    EVT_BUTTON(wxID_RETRY,TNoReceiverFoundWindow::OnTryAgain)
    EVT_BUTTON(wxID_DEFAULT,TNoReceiverFoundWindow::OnDemo)
    EVT_BUTTON(wxID_MORE,TNoReceiverFoundWindow::OnScanNetwork)
END_EVENT_TABLE()

TReceiverSelector::TReceiverSelector(IG69DDCDeviceEnumerator *Enumerator):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition)
{
 uint32_t i;
 G69DDC_DEVICE_INFO DeviceInfo;
 wxString *list;
 wxPanel *p;

    SetFont(*Global.m_Font);

    p=new wxPanel(this,wxID_ANY,wxPoint(0,0),wxSize(100,100));

    Layout();

    list=new wxString[Enumerator->GetCount()];

    for(i=0;i<Enumerator->GetCount();i++)
    {
        Enumerator->GetDeviceInfo(i,&DeviceInfo);

        list[i].sprintf(wxT("%s"),wxString(DeviceInfo.SerialNumber,wxConvLocal));
    }

    m_RadioBox=new wxRadioBox(p,wxID_ANY,wxT("Select receiver to open"),wxPoint(20,20),wxSize(130,Enumerator->GetCount()*20+25),Enumerator->GetCount(),list,0,wxVERTICAL);

    delete[] list;

    m_BOK=new wxButton(p,wxID_OK,wxT("OK"),wxPoint(165,20),wxSize(70,23));

    m_BCancel=new wxButton(p,wxID_CANCEL,wxT("Cancel"),wxPoint(165,50),wxSize(70,23));

    m_SelectedReceiverIndex=m_RadioBox->GetSelection();

    SetClientSize(m_BOK->GetRect().GetRight()+20,m_RadioBox->GetRect().GetBottom()+20);

    CenterOnScreen();
}

TReceiverSelector::~TReceiverSelector(void)
{
}

void TReceiverSelector::OnOk(wxCommandEvent &Event)
{
    m_SelectedReceiverIndex=m_RadioBox->GetSelection();
    EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(TReceiverSelector, wxDialog)
    EVT_BUTTON(wxID_OK,TReceiverSelector::OnOk)
END_EVENT_TABLE()


TReceiver* TReceiverSelector::SelectReceiver(void)
{
 IG69DDCDeviceEnumerator *Enumerator;
 TNoReceiverFoundWindow *NoReceiver;
 TReceiver *Receiver;
 TReceiverSelector *Selector;
 TScanNetwork *ScanNetwork;
 uint32_t Count;
 int r;
 G69DDC_DEVICE_INFO DeviceInfo;
 wxString Address;

    if(!CreateInstance(G69DDC_CLASS_ID_DEVICE_ENUMERATOR,(void**)&Enumerator))
    {
        wxMessageBox(wxT("Failed to create device enumerator object!"),APP_TITLE,wxICON_ERROR);
        return NULL;
    }

    Receiver=new TReceiver;

    for(;;)
    {
        Count=Enumerator->GetCount();

        if(Count==0) //no receiver has been found
        {
            NoReceiver=new TNoReceiverFoundWindow;

            r=NoReceiver->ShowModal();

            delete NoReceiver;

            if(r==wxID_RETRY)
            {
                Enumerator->Enumerate();
            }
            else if(r==wxID_DEFAULT)
            {
                if(Receiver->Open(G69DDC_OPEN_DEMO))
                {
                    Enumerator->Free();
                    return Receiver;
                }

                wxMessageBox(wxT("Failed to open demo receiver!"),APP_TITLE,wxICON_ERROR);
                break;
            }
            else if(r==wxID_MORE)
            {
                ScanNetwork=new TScanNetwork;

                r=ScanNetwork->ShowModal();

                if(r==wxID_OK)
                {
                    if(Receiver->Open(ScanNetwork->GetAddress().c_str()))
                    {
                        delete ScanNetwork;
                        return Receiver;
                    }
                }

                delete ScanNetwork;

                Enumerator->Enumerate();
            }
            else
            {
                break;
            }
        }
        else if(Count==1) //single receiver has been found
        {
            Enumerator->GetDeviceInfo(0,&DeviceInfo);

            if(Receiver->Open(DeviceInfo.DevicePath))
            {
                //receiver has been open
                Enumerator->Free();
                return Receiver;
            }

            wxMessageBox(wxT("Failed to open receiver!"),APP_TITLE,wxICON_ERROR);
            break;
        }
        else
        {
            //more than one receiver has been detected
            Selector=new TReceiverSelector(Enumerator);

            if(Selector->ShowModal()!=wxID_OK)
            {
                delete Selector;
                break;
            }

            r=Selector->GetSelectedReceiverIndex();

            delete Selector;

            Enumerator->GetDeviceInfo(r,&DeviceInfo);

            if(Receiver->Open(DeviceInfo.DevicePath))
            {
                Enumerator->Free();
                return Receiver;
            }

            wxMessageBox(wxT("Failed to open receiver!"),APP_TITLE,wxICON_ERROR);
            break;
        }
    }

    Enumerator->Free();
    delete Receiver;

    return NULL;
}

void TNoReceiverFoundWindow::OnScanNetwork(wxCommandEvent &Event)
{
    EndModal(wxID_MORE);
}

enum
{
    ID_TIMER=wxID_HIGHEST,
};

wxDEFINE_EVENT(EVT_FOUND_DEVICE_LIST_CHANGED, wxCommandEvent);

BEGIN_EVENT_TABLE(TScanNetwork, wxDialog)
    EVT_BUTTON(wxID_OK,TScanNetwork::OnOk)
    EVT_BUTTON(wxID_CANCEL,TScanNetwork::OnCancel)
    EVT_TIMER(ID_TIMER,TScanNetwork::OnTimer)
    EVT_COMMAND(wxID_ANY,EVT_FOUND_DEVICE_LIST_CHANGED,TScanNetwork::OnDeviceListChanged)
    EVT_LIST_ITEM_DESELECTED(wxID_ANY,TScanNetwork::OnDeselected)
    EVT_LIST_ITEM_SELECTED(wxID_ANY,TScanNetwork::OnSelected)
END_EVENT_TABLE()


TScanNetwork::TScanNetwork(void):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition),m_Mutex(wxMUTEX_RECURSIVE)
{
 wxSizer *Sizer,*ButtonSizer;
 wxStaticText *Text;


    SetFont(*Global.m_Font);

    CreateInstance(G69DDC_CLASS_ID_NETWORK_ENUMERATOR,(void**)&m_Enumerator);

    Sizer=new wxBoxSizer(wxVERTICAL);

    Text=new wxStaticText(this,wxID_ANY,wxT("Select receiver to open"));
    Sizer->Add(Text,0,wxEXPAND|wxALIGN_TOP|wxTOP|wxLEFT|wxRIGHT,20);

    m_LV=new wxListView(this,wxID_ANY,wxPoint(20,20),wxSize(200,100),wxLC_REPORT|wxLC_SINGLE_SEL);
    m_LV->AppendColumn(wxT("Serial number"),wxLIST_FORMAT_LEFT,80);
    m_LV->AppendColumn(wxT("Remote address"),wxLIST_FORMAT_LEFT,150);

    Sizer->Add(m_LV,1,wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM,20);


    ButtonSizer=new wxBoxSizer(wxHORIZONTAL);
    Sizer->Add(ButtonSizer,0,wxALIGN_CENTER|wxALIGN_TOP|wxBOTTOM,20);


    m_BOK=new wxButton(this,wxID_OK,wxT("OK"),wxDefaultPosition,wxSize(80,23));
    ButtonSizer->Add(m_BOK,0,wxALIGN_CENTER);
    m_BOK->Enable(false);

    ButtonSizer->AddSpacer(4);

    m_BCancel=new wxButton(this,wxID_CANCEL,wxT("Cancel"),wxDefaultPosition,wxSize(80,23));
    ButtonSizer->Add(m_BCancel,0,wxALIGN_CENTER);


    SetSizer(Sizer);


    SetClientSize(300,300);
    CenterOnScreen();

    m_Timer.SetOwner(this,ID_TIMER);

    m_Enumerator->Start(this);

    m_Timer.Start(1000);
}

TScanNetwork::~TScanNetwork(void)
{
    m_Enumerator->Free();
}

void TScanNetwork::G69DDC_DeviceFound(uint32_t IpAddress,uint16_t Port,const char *SerialNumber,uint32_t Flags)
{
 TDeviceInfo *DeviceInfo;

    if(Flags & 0x04)
    {
        //the device is locked by another client, for more information, see IG69DDCNetworkEnumeratorCallback::G69DDC_DeviceFound in the G69DDC SDK documentation
        return;
    }

    DeviceInfo=new TDeviceInfo(SerialNumber,IpAddress,Port);

    m_Mutex.Lock();

    m_FoundDevices.push_back(DeviceInfo);

    if(m_FoundDevices.size()==1)
    {
        wxCommandEvent event(EVT_FOUND_DEVICE_LIST_CHANGED);
        wxPostEvent(this,event);
    }

    m_Mutex.Unlock();
}

void TScanNetwork::OnTimer(wxTimerEvent& Event)
{
 wxUIntPtr Data;

    m_Enumerator->Start(this);

    for(int i=m_LV->GetItemCount()-1;i>=0;i--)
    {
        Data=m_LV->GetItemData(i);

        if(Data>0)
        {
            m_LV->SetItemData(i,Data-1);
        }
        else
        {
            m_LV->DeleteItem(i);
        }
    }
}

void TScanNetwork::OnDeviceListChanged(wxCommandEvent &Event)
{
 wxVector<TDeviceInfo*> FoundDevices;
 wxString s;
 long Item;
 uint32_t IpAddress;

    m_Mutex.Lock();

    m_FoundDevices.swap(FoundDevices);

    m_Mutex.Unlock();

    for(size_t i=0;i<FoundDevices.size();i++)
    {
        Item=m_LV->FindItem(-1,FoundDevices[i]->m_SerialNumber);

        if(Item==wxNOT_FOUND)
        {
            Item=m_LV->GetItemCount();

            IpAddress=ntohl(FoundDevices[i]->m_IpAddress);

            s.sprintf(wxT("%u.%u.%u.%u:%u"),(IpAddress>>24)&0xFF,(IpAddress>>16)&0xFF,(IpAddress>>8)&0xFF,IpAddress&0xFF,ntohs(FoundDevices[i]->m_Port));

            Item=m_LV->InsertItem(Item,FoundDevices[i]->m_SerialNumber);
            m_LV->SetItem(Item,1,s);
        }

        m_LV->SetItemData(Item,1);

        delete FoundDevices[i];
    }
}

void TScanNetwork::OnOk(wxCommandEvent &Event)
{
    if(m_LV->GetSelectedItemCount())
    {
        long Item=m_LV->GetNextItem(-1,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);

        if(Item>=0)
        {
            m_Address=wxT("udp:")+m_LV->GetItemText(Item,1); //convert address to requered format: "udp:ADDRESS:PORT", see IG69DDCDevice::Open in SDK documentation
            EndModal(wxID_OK);
        }
    }
}

void TScanNetwork::OnCancel(wxCommandEvent &Event)
{
    EndModal(wxID_CANCEL);
}

void TScanNetwork::OnDeselected(wxListEvent& Event)
{
    if(m_LV->GetSelectedItemCount()==0)
    {
        m_BOK->Enable(false);
    }
}

void TScanNetwork::OnSelected(wxListEvent& Event)
{
    if(m_LV->GetSelectedItemCount())
    {
        m_BOK->Enable(true);
    }
}

